package com.example.controller;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.json.JSONUtil;
import com.example.common.Result;
import com.example.entity.SaleStatement;
import com.example.service.SaleStatementService;
import com.example.utils.DateHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;

/**
 * SaleStatement销量表接口
 **/
@RestController
@RequestMapping("/saleStatement")
public class SaleStatementController {
    @Resource
    private SaleStatementService saleStatementService;

    /**
     * 商品售出完成时间和商品id进行报表查询
     * @param productId
     * @param timeStart
     * @param timeOver
     * @return 该商品在timeStart~timeOver之间的销售量，销售额度
     */
    @GetMapping("/getByPId")
    public Result getByPId(Integer productId,String timeStart, String timeOver){
        SaleStatement pre = new SaleStatement();
        pre.setTimeOver(timeOver);
        pre.setTimeStart(timeStart);
        pre.setProductId(productId);

        return Result.success(saleStatementService.selectByRange(pre));
    }

    /**
     * 批量获取
     * @return
     */
    @PostMapping("/getByPIdBatch")
    public Result getByPIdBatch(@RequestBody SaleStatement saleStatement){
        // 批量获取
        return Result.success(saleStatementService.selectByRangeBatch(saleStatement.getProductIds(),saleStatement));
    }
    /**
     * 获取所有时间内的报表信息
     * @param timeStart 起始时间
     * @param timeOver 截止时间
     * @return
     */
    @GetMapping("/getAllP")
    public Result getByAllP(String timeStart, String timeOver){
        SaleStatement pre = new SaleStatement();
        pre.setTimeOver(timeOver);
        pre.setTimeStart(timeStart);

        return Result.success(saleStatementService.selectByRange(pre));
    }
    /**
     * 获取店铺销当前所有季度的销售额
     * Finished
     */
    @PostMapping("/getAllSeasonSale")
    public Result getAllSeasonSale(@RequestBody SaleStatement saleStatement){
        // 依次获取当年1~当前月份的销售额度
        List<String> beforeCurYearMonth = DateHelper.getBeforeCurYear_Month(DateUtil.month(DateUtil.date()));
        List<SaleStatement> monthSaleStatement = new LinkedList<>();
        // mybatis获取所有月份的总消费
        List<BigDecimal> resSeason = new ArrayList<>();
        // 根据月份时间查询当月的所有销售额
        for(int i = 0 ;i<beforeCurYearMonth.size();i++){

            String computedYM = beforeCurYearMonth.get(i) ;
            String YM_StartDate = computedYM + "-01";
            String YM_EndDate = computedYM + "-31";
            saleStatement.setTimeStart(YM_StartDate);
            saleStatement.setTimeOver(YM_EndDate);

            // 防止空
            SaleStatement dbSatement = saleStatementService.selectAllByRnge(saleStatement);
            if(dbSatement==null){
                dbSatement = new SaleStatement();
                dbSatement.setAmount(new BigDecimal(0));
            }
            monthSaleStatement.add(dbSatement);
        }
        // 一共x个月
        int len = monthSaleStatement.size();
        // 第i季度的消费
        Map<String ,SaleStatement> seasonMap = new HashMap<>();
        SaleStatement temp = new SaleStatement();
        int season = 1;
        boolean flag [] = {false,false,false,false,false};
        for(int curMonth = 0; curMonth<len;curMonth++){
            if(temp.getAmount()==null){
                temp.setAmount(new BigDecimal(0));
            }
            if(curMonth ==(season*3-1) ){
                flag[season] = true;
                // 添加到新季度中
                resSeason.add(temp.getAmount());
                season++;
                temp = new SaleStatement();
                temp.setAmount(new BigDecimal(0));
            }
            BigDecimal newAmount = monthSaleStatement.get(curMonth).getAmount().add(temp.getAmount());
            temp.setAmount(newAmount);
        }
        if(!flag[season]){ // 季度没有算完的补充计算
            resSeason.add(temp.getAmount());
        }

        // 构建返回值
        Map<String ,Object> res = new HashMap<>();
        List<BigDecimal> resMonth = new ArrayList<>();
        monthSaleStatement.forEach(dbStatement->{resMonth.add(dbStatement.getAmount());});

        res.put("monthAccount",resMonth);
        res.put("seasonAccount",resSeason);
        return  Result.success(res);
    }

    /**
     * 获取店铺所有产品的销售信息
     * TODO
     * @return
     */
    @GetMapping("/getShopPdAccount/{shopId}")
    public Result getShopPdAccount(@PathVariable Integer shopId,String timeStart,String timeOver){
        return Result.success(saleStatementService.getShopPdAccount(shopId,timeStart,timeOver));
    }
    /**
     * 新增
     */
    @PostMapping("/add")
    public Result add(@RequestBody SaleStatement saleStatement) {
        saleStatementService.add(saleStatement);
        return Result.success();
    }

    /**
     * 删除
     */
    @DeleteMapping("/delete/{id}")
    public Result deleteById(@PathVariable Integer id) {
        saleStatementService.deleteById(id);
        return Result.success();
    }

    /**
     * 批量删除
     */
    @DeleteMapping("/delete/batch")
    public Result deleteBatch(@RequestBody List<Integer> ids) {
        saleStatementService.deleteBatch(ids);
        return Result.success();
    }

    /**
     * 修改
     */
    @PutMapping("/update")
    public Result updateById(@RequestBody SaleStatement saleStatement) {
        saleStatementService.updateById(saleStatement);
        return Result.success();
    }

    /**
     * 根据ID查询
     */
    @GetMapping("/selectById/{id}")
    public Result selectById(@PathVariable Integer id) {
        SaleStatement saleStatement = saleStatementService.selectById(id);
        return Result.success(saleStatement);
    }

    /**
     * 查询所有
     */
    @GetMapping("/selectAll")
    public Result selectAll(SaleStatement saleStatement) {
        List<SaleStatement> list = saleStatementService.selectAll(saleStatement);
        return Result.success(list);
    }

    /**
     * 分页查询
     */
    @GetMapping("/selectPage")
    public Result selectPage(SaleStatement saleStatement,
                             @RequestParam(defaultValue = "1") Integer pageNum,
                             @RequestParam(defaultValue = "10") Integer pageSize) {
        PageInfo<SaleStatement> page = saleStatementService.selectPage(saleStatement, pageNum, pageSize);
        return Result.success(page);
    }

    /**
     *
     * @param id
     * @param day
     * @return
     */
    @RequestMapping(
            path = {"/{id}/{day}"},
            method = {RequestMethod.GET}
    )
    public Result selectByDay(@PathVariable Integer id ,@PathVariable Integer day){

        return Result.success(id + " "+day);
    }
}