package com.example.service;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.example.entity.SaleStatement;
import com.example.mapper.SaleStatementMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import javax.annotation.Resource;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;

/**
 * 销量表业务处理
 **/
@Service
public class SaleStatementService {

    @Resource
    private SaleStatementMapper saleStatementMapper;
    @Resource
    private StoreService storeService;
    /**
     * 筛选特定时间内的商品销售额和销售量
     */
    public SaleStatement selectByRange(SaleStatement saleStatement){
        return  saleStatementMapper.selectOneProductSalementAndAccount(saleStatement);
    }

    /**
     * 获取时间内的所有销量金额
     * @param saleStatement
     * @return
     */
    public SaleStatement selectAllByRnge(SaleStatement saleStatement){
        return saleStatementMapper.selectAllProductsAccount(saleStatement);
    }
    public List<SaleStatement> selectByRangeBatch(List<Integer> ids , SaleStatement saleStatement){
        // 依次读取
        List<SaleStatement> res = new ArrayList<SaleStatement>();
        ids.forEach(id ->{
            saleStatement.setProductId(id);
            res.add(saleStatementMapper.selectOneProductSalementAndAccount(saleStatement));
        });
        return res;
    }

    /**
     * 获取商店指定商品的销售额
      * @param shopId
     * @param timeStart
     * @param timeOver
     * @return
     */
    public List<HashMap<String, Object>> getShopPdAccount(Integer shopId,String timeStart,String timeOver){
        // 获取店铺产品
        List<Integer> storeProductIds = storeService.getStoreProductIds(shopId);
        // 获取对应店铺报表
        // 根据商品id查找时间范围内的商品信息
        // 查询产品信息
        Map<String ,Object> retMap = new HashMap<>();
        List<HashMap<String, Object>> retList = new ArrayList<>(); // 返回的数据
        storeProductIds.forEach(id->{
            SaleStatement saleStatement = new SaleStatement();
            saleStatement.setShopId(shopId);
            saleStatement.setProductId(id);
            saleStatement.setTimeStart(timeStart);
            saleStatement.setTimeOver(timeOver);
            SaleStatement saleStatement1 = saleStatementMapper.selectProductSalementInfoRange(saleStatement);
            if(saleStatement1!=null){
                HashMap<String, Object> item = new HashMap<>();
                item.put("productName",saleStatement1.getProductName());
                item.put("amount",saleStatement1.getAmount());
                item.put("count",saleStatement1.getCount());
                retList.add(item);
            }

        });
        return retList;
    }

    /**
     * 新增
     */
    public void add(SaleStatement saleStatement) {
        saleStatementMapper.insert(saleStatement);
    }

    /**
     * 删除
     */
    public void deleteById(Integer id) {
        saleStatementMapper.deleteById(id);
    }

    /**
     * 批量删除
     */
    public void deleteBatch(List<Integer> ids) {
        for (Integer id : ids) {
            saleStatementMapper.deleteById(id);
        }
    }

    /**
     * 修改
     */
    public void updateById(SaleStatement saleStatement) {
        saleStatementMapper.updateById(saleStatement);
    }

    /**
     * 根据ID查询
     */
    public SaleStatement selectById(Integer id) {
        return saleStatementMapper.selectById(id);
    }

    /**
     * 查询所有
     */
    public List<SaleStatement> selectAll(SaleStatement saleStatement) {
        return saleStatementMapper.selectAll(saleStatement);
    }

    /**
     * 分页查询
     */
    public PageInfo<SaleStatement> selectPage(SaleStatement saleStatement, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<SaleStatement> list = saleStatementMapper.selectAll(saleStatement);
        return PageInfo.of(list);
    }

}