package com.example.service;

import com.example.entity.Product;
import com.example.entity.ProductType;
import com.example.entity.Store;
import com.example.mapper.StoreMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import javax.annotation.Resource;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 业务处理
 **/
@Service
public class StoreService {

    @Resource
    private StoreMapper storeMapper;

    /**
     * 新增
     */
    public void add(Store store) {
        storeMapper.insert(store);
    }

    /**
     * 删除
     */
    public void deleteById(Integer id) {
        storeMapper.deleteById(id);
    }

    /**
     * 批量删除
     */
    public void deleteBatch(List<Integer> ids) {
        for (Integer id : ids) {
            storeMapper.deleteById(id);
        }
    }

    /**
     * 修改
     */
    public void updateById(Store store) {
        storeMapper.updateById(store);
    }

    /**
     * 根据ID查询
     */
    public Store selectById(Integer id) {
        return storeMapper.selectById(id);
    }

    /**
     * 查询所有
     */
    public List<Store> selectAll(Store store) {
        return storeMapper.selectAll(store);
    }

    /**
     * 分页查询
     */
    public PageInfo<Store> selectPage(Store store, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<Store> list = storeMapper.selectAll(store);
        return PageInfo.of(list);
    }

    /**
     * shopId获取店铺商品类型
     * @param shopId
     * @return
     */
    public List<ProductType> selectProductsTypesByShopId(Integer shopId) {
        return storeMapper.selectProductsTypes(shopId).getProductTypes();
    }

    /**
     * 获取店铺产品
     * @param storeId
     * @return
     */
    public List<Store> selectProducts(Integer storeId){
        Store store = new Store();
        store.setId(storeId);
//        List<Product> products = storeMapper.selectAll(store);
        return  storeMapper.selectAll(store);
    }

    public void updateProductIdsWithId(Integer storeId,Integer productId){
        Store store = storeMapper.selectById(storeId);
        Integer[] productsIds = store.getProductsIds();
        Set<Integer> ids =Arrays.stream(productsIds).collect(Collectors.toSet());
        ids.add(productId);
        Integer[] productIds = ids.toArray(new Integer[0]);
        store.setProductsIds(productIds);
        storeMapper.updateById(store);
    }
    public List<Integer> getStoreProductIds(Integer storeId){
        if(storeMapper.selectStorePIds(storeId)!=null)
        return List.of(storeMapper.selectStorePIds(storeId).getProductsIds());
        return null;
    }
}